/*
	CameraEventFactory
	
	Makes a bunch of mock objects; fake CameraEventVO's.
	Since it can take awhile, he does it asynchronously.

*/

package
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.utils.getTimer;
	
	import mx.collections.ArrayCollection;
	import mx.managers.CursorManager;
	
	
	[Event(name="completedProcessing", type="FactoryEvent")]
	public class CameraEventFactory extends EventDispatcher
	{
		private static const EVENT_RANGE_SECONDS:Number = 7; // events are usually a minimum of 3, and a maximum of 10 seconds
		private static const GAP_RANGE_SECONDS:Number	= 30; // maximum range between events
		
		private var tickSprite:Sprite;
		private var currentIndex:int;
		private var total:uint;
		private var incrementDate:Date;
		private var list:Array;
		private var index:int;
		private var len:int;
		private var start:int;
		
		public function getRandomEvents(amount:uint, startDate:Date):void
		{
			start				= getTimer();
			CursorManager.setBusyCursor();
			currentIndex 		= -1;
			total 				= amount;
			incrementDate 		= new Date(startDate.valueOf());
			list 				= [];
			processNext();
		}
		
		private function processNext():void
		{
			if(currentIndex + 1 < total)
			{
				var st:int 						= getTimer();
				var startDate:Date;
				var endDate:Date;
				const TIMEOUT:int				= 24;
				for(index = currentIndex; index < total; index++)
				{
					startDate 					= new Date(incrementDate.valueOf());
					endDate 					= new Date(startDate.valueOf());
					endDate.seconds		 		+= Math.floor(EVENT_RANGE_SECONDS * Math.random()) + 3;
					incrementDate				= new Date(endDate.valueOf());
					incrementDate.seconds	 	+= Math.floor(Math.random() * GAP_RANGE_SECONDS) + 3;
					list[index] 				= new CameraEventVO(new Date(startDate.valueOf()), new Date(endDate.valueOf()), Math.floor(Math.random() * 255));
					if(getTimer() - st > TIMEOUT)
					{
						currentIndex = index;
						if(tickSprite == null) tickSprite = new Sprite();
						tickSprite.addEventListener(Event.ENTER_FRAME, onBreaksOver);
						return;
					}
				}
			}
			CursorManager.removeBusyCursor();
			var evt:FactoryEvent = new FactoryEvent(FactoryEvent.COMPLETED_PROCESSING);
			evt.cameraEvents = new ArrayCollection(list);
			evt.totalProcessingTime = getTimer() - start;
			dispatchEvent(evt);
		}
		
		private function onBreaksOver(event:Event):void
		{
			tickSprite.removeEventListener(Event.ENTER_FRAME, onBreaksOver);
			processNext();
		}
	
		private static var _inst:CameraEventFactory;
		
		public static function get instance():CameraEventFactory
		{
			if(_inst == null) _inst = new CameraEventFactory();
			return _inst;
		}	
	}
}