/*

	SimpleGraph
	
	This simple bar graph draws CameraEventVO's over time.
	This ensures you can put as many as you want into it,
	as well as add more later, and she'll scale.
	
	...unfortunately, there are limits.  While I could
	write a dynamic bitmap creator, I didn't, thus,
	you only get 1, therefore, the maximum width is
	2880 in Flash Player 9, or around 8000 in 10.
	
	The startDate and endDate determine size of the graph.
	The viewStartDate determines what portion of the graph you
	are viewing since it can be pretty big in width.


*/
package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.CapsStyle;
	import flash.display.GradientType;
	import flash.display.Graphics;
	import flash.display.JointStyle;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.filters.DropShadowFilter;
	import flash.geom.Matrix;
	import flash.geom.Rectangle;
	import flash.system.Capabilities;
	import flash.utils.getTimer;
	
	import mx.collections.ArrayCollection;
	import mx.controls.Label;
	import mx.core.UIComponent;
	import mx.events.CollectionEvent;
	import mx.events.CollectionEventKind;
	import mx.formatters.DateFormatter;

	public class SimpleGraph extends UIComponent
	{
		private var _cameraEvents:ArrayCollection;
		private var needToRedraw:Boolean = false;
		private var _startDate:Date;
		private var _endDate:Date;
		private var _viewStartDate:Date;
		private var viewStartDateDirty:Boolean = false;
		private var viewEndDate:Date;
		private var range:Number; // milliseconds
		private var datesDirty:Boolean = false;
		private var appendDirty:Boolean = false;
		private var currentDrawIndex:uint;
		private var newIndex:int;
		private var graphBorder:Rectangle;
		private var dateFormatter:DateFormatter;
		
		private var background:Shape;
		private var lense:Shape;
		private var tickBitmap:Bitmap;
		private var redrawSprite:Sprite;
		private var tickMask:Shape;
		private var startDateLabel:Label;
		private var endDateLabel:Label;
		
		// our list of CameraEventVO's
		public function get cameraEvents():ArrayCollection { return _cameraEvents; }
		public function set cameraEvents(value:ArrayCollection):void
		{
			if(_cameraEvents)
			{
				_cameraEvents.removeEventListener(CollectionEvent.COLLECTION_CHANGE, onChanged);
			}
			_cameraEvents = value;
			if(_cameraEvents)
			{
				_cameraEvents.addEventListener(CollectionEvent.COLLECTION_CHANGE, onChanged);
			}
			needToRedraw = true;
			invalidateProperties();
		}
		
		public function get startDate():Date { return _startDate; }
		public function set startDate(value:Date):void
		{
			_startDate = value;
			if(_startDate)
			{
				viewStartDate = new Date(_startDate.valueOf());
			}
			else
			{
				viewStartDate = null;
			}
			calculateDateRange();
			datesDirty = true;
			invalidateProperties();
		}
		
		public function get endDate():Date { return _endDate; }
		public function set endDate(value:Date):void
		{
			_endDate = value;
			if(_endDate && startDate && viewStartDate)
			{
				viewEndDate = new Date(_viewStartDate.valueOf() + (endDate.valueOf() - startDate.valueOf()));
			}
			else
			{
				viewEndDate = null;
			}
			calculateDateRange();
			datesDirty = true;
			invalidateProperties();
		}
		
		// what portion of the graph we're viewing
		public function get viewStartDate():Date { return _viewStartDate; }
		public function set viewStartDate(value:Date):void
		{
			_viewStartDate = value;
			if(_viewStartDate && startDate && endDate)
			{
				viewEndDate = new Date(_viewStartDate.valueOf() + (endDate.valueOf() - startDate.valueOf()));
			}
			else
			{
				viewEndDate = null;
			}
			viewStartDateDirty = true;
			invalidateProperties();
		}
		
		public function SimpleGraph()
		{
			super();
			
			addEventListener("widthChanged", onSizeChanged);
			addEventListener("heightChanged", onSizeChanged);
			
			dateFormatter = new DateFormatter();
			dateFormatter.formatString = "MMMM, D";
			
			width			= 400;
			height			= 72;
		}
		
		protected override function createChildren():void
		{
			background = new Shape();
			addChild(background);
			var drop:DropShadowFilter = new DropShadowFilter(2, 135, 0x333333, .4, 3, 3, 1, 1, false, true, false);
			background.filters = [drop];
			
			tickBitmap = new Bitmap();
			addChild(tickBitmap);
			
			lense = new Shape();
			addChild(lense);
			lense.y = 2;
			
			tickMask = new Shape();
			addChild(tickMask);
			redrawTickMask();
			
			startDateLabel = new Label();
			addChild(startDateLabel);
			startDateLabel.truncateToFit = false;
			startDateLabel.setStyle("fontFamily", "Verdana");
			startDateLabel.setStyle("fontSize", 9);
			startDateLabel.setStyle("color", 0x333333);
			
			endDateLabel = new Label();
			addChild(endDateLabel);
			endDateLabel.truncateToFit = false;
			endDateLabel.setStyle("fontFamily", "Verdana");
			endDateLabel.setStyle("fontSize", 9);
			endDateLabel.setStyle("color", 0x333333);
			endDateLabel.setStyle("textAlign", "right");
		}
		
		protected override function commitProperties():void
		{
			super.commitProperties();
			
			if(datesDirty)
			{
				datesDirty = false;
				needToRedraw = false;
				appendDirty = false; // we can't appende because we're redrawing EVERYTHING, ZOMG
				redraw();
			}
			
			if(needToRedraw)
			{
				needToRedraw = false;
				appendDirty = false; // we can't appende because we're redrawing EVERYTHING, ZOMG
				redraw();
			}
			
			if(appendDirty)
			{
				appendDirty = false;
				drawNew();
			}
			
			if(viewStartDateDirty)
			{
				viewStartDateDirty 		= false;
				startDateLabel.text 	= dateFormatter.format(viewStartDate);
				endDateLabel.text 		= dateFormatter.format(viewEndDate);
				updateView();
			}
			
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var g:Graphics = background.graphics;
			g.clear();
			g.lineStyle(1, 0x333333, .4, true, "normal", CapsStyle.ROUND, JointStyle.ROUND);
			g.beginFill(0xEEEEEE, .8);
			g.drawRoundRect(0, 0, unscaledWidth, unscaledHeight, 12, 12);
			g.endFill();
			
			var box:Matrix = new Matrix();
			var half:Number = unscaledHeight / 2;
			box.createGradientBox(unscaledWidth, half, Math.PI / 2, 0, 0);
			g = lense.graphics;
			g.clear();
			g.beginGradientFill(GradientType.LINEAR, [0xFFFFFF, 0xFFFFFF], [.8, 0], [0, 255], box);
			g.drawRoundRect(0, 0, unscaledWidth, half, 12, 12);
			g.endFill();
			
			const MARGIN:Number = 2;
			startDateLabel.setActualSize(startDateLabel.textWidth + 4, startDateLabel.textHeight + 4);
			startDateLabel.move(graphBorder.x, height - (startDateLabel.textHeight + 4));
			
			endDateLabel.setActualSize(endDateLabel.textWidth + 4, endDateLabel.textHeight + 4);
			endDateLabel.move(width - (endDateLabel.textWidth + 4) - ((width - graphBorder.width) / 2), height - (endDateLabel.textHeight + 4) + MARGIN);
			
		}
		
		private function onSizeChanged(event:Event):void
		{
			needToRedraw = true;
			var w:Number = Math.max(9, width);
			var h:Number = Math.max(9, height);
			const MARGIN:Number = 8;
			const MARGIN2:Number = 16;
			const BOTTOM_MARGIN:Number = 24;
			if(graphBorder)
			{
				graphBorder.width = w - MARGIN2;
				graphBorder.height = h - BOTTOM_MARGIN;
			}
			else
			{
				graphBorder = new Rectangle(MARGIN, MARGIN, w - MARGIN2, h - BOTTOM_MARGIN);
			}
			redrawTickMask();
			invalidateProperties();
		}
		
		private function redrawTickMask():void
		{
			if(tickMask)
			{
				var g:Graphics = tickMask.graphics;
				g.clear();
				g.beginFill(0x00FF00, .5);
				g.drawRect(graphBorder.x, graphBorder.y, graphBorder.width, graphBorder.height);
				g.endFill();
				tickBitmap.mask = tickMask;
			}
		}
		
		// recreate our bitmap, and prepare to draw
		private function redraw():void
		{
			if(redrawSprite) redrawSprite.removeEventListener(Event.ENTER_FRAME, onBreakIsOvaSucka);
			if(startDate && endDate && cameraEvents && cameraEvents.length > 0)
			{
				if(tickBitmap.bitmapData) tickBitmap.bitmapData.dispose();
				var list:Array 		= Capabilities.version.split(" ");
				var digits:Array 	= list[1].split(",");
				var v:uint 			= parseInt(digits[0]);
				if(v >= 10)
				{
					tickBitmap.bitmapData = new BitmapData(8000, height, true, 0);
				}
				else
				{
					tickBitmap.bitmapData = new BitmapData(2880, height, true, 0);
				}
				tickBitmap.x = 0;
				currentDrawIndex = 0;
				drawNext();
			}
		}
		
		private function drawNew():void
		{
			if(redrawSprite) return; // abort, our drawing function uses that sprite, so let's not draw twice
			if(startDate && endDate && cameraEvents && cameraEvents.length > 0 && isNaN(newIndex) == false)
			{
				currentDrawIndex = newIndex;
				drawNext();
			}
		}
		
		// draws as many CameraEventVO's as possible within the timeslice we're given
		private function drawNext():void
		{
			var st:int = getTimer();
			if(startDate && endDate && cameraEvents && cameraEvents.length > 0 && currentDrawIndex + 1 < cameraEvents.length)
			{
				var len:int = cameraEvents.length;
				const TIMEOUT:uint					= 5;
				tickBitmap.bitmapData.lock();
				for(var index:uint = currentDrawIndex; index < len; index++)
				{
					var cameraEvent:CameraEventVO 	= cameraEvents.getItemAt(index) as CameraEventVO;
					drawEvent(cameraEvent);
					currentDrawIndex++;
					if(getTimer() - st > TIMEOUT)
					{
						break;
					}
				}
				// if we're not done, wait a frame
				if(currentDrawIndex + 1 < cameraEvents.length)
				{
					if(redrawSprite == null) redrawSprite = new Sprite();
					redrawSprite.addEventListener(Event.ENTER_FRAME, onBreakIsOvaSucka);
					return;
				}
			}
			
			if(redrawSprite)
			{
				redrawSprite.removeEventListener(Event.ENTER_FRAME, onBreakIsOvaSucka);
				redrawSprite = null;
			}
			tickBitmap.bitmapData.unlock();
		}
		
		// draw a bar from a CameraEventVO
		private function drawEvent(cameraEvent:CameraEventVO):void
		{
			var targetX:Number 				= DateUtils.getXFromDate(startDate, endDate, cameraEvent.startTime, graphBorder.width);
			var endX:Number					= DateUtils.getXFromDate(startDate, endDate, cameraEvent.endTime, graphBorder.width);
			var drawWidth:Number			= endX - targetX;
			var lineHeight:Number			= (cameraEvent.motionLevel / CameraEventVO.MAX_MOTION_LEVEL) * graphBorder.height;
			for(var r:uint = targetX; r < targetX + drawWidth; r++)
			{
				for(var c:uint = graphBorder.y + lineHeight; c < graphBorder.height - lineHeight; c++)
				{
					tickBitmap.bitmapData.setPixel32(r, c, 0xFF660000);
				}
			}
		}
		
		// 1 frame later, we've given the screen time to update
		// although, not necessarely given Flash enough time to breath...
		private function onBreakIsOvaSucka(event:Event):void
		{
			redrawSprite.removeEventListener(Event.ENTER_FRAME, onBreakIsOvaSucka);
			drawNext();
		}
		
		// if a new event is added, attempt to redraw next frame
		private function onChanged(event:CollectionEvent):void
		{
			switch(event.kind)
			{
				case CollectionEventKind.ADD:
					// ensure we're not in the middle of drawing
					if(redrawSprite == null)
					{
						if(isNaN(newIndex))
						{
							newIndex = event.location;
						}
						else
						{
							newIndex = Math.min(newIndex, event.location);
						}
						appendDirty = true;
						invalidateProperties();
					}
					break;
			}
		}
		
		private function calculateDateRange():void
		{
			if(startDate && endDate)
			{
				range = endDate.valueOf() - startDate.valueOf();
			}
			else
			{
				range = 0;
			}
		}
		
		// scroll the graph based on our viewing date
		private function updateView():void
		{
			if(startDate && endDate && viewStartDate && width)
			{
				var targetX:Number 	= DateUtils.getXFromDate(startDate, endDate, viewStartDate, width);
				tickBitmap.x 		= -1 * targetX;
			}
		}
		
	}
}